#pragma once

#include "base.h"
#include "datatypes/cframe.h"
#include "objects/annotation.h"
#include "objects/base/service.h"
#include "objects/part.h"
#include <array>
#include <memory>
#include <reactphysics3d/body/RigidBody.h>

class HandleFace {
    HandleFace(int index, glm::vec3 normal) : index(index), normal(normal){}

    public:
    int index;
    glm::vec3 normal;

    static HandleFace XPos;
    static HandleFace XNeg;
    static HandleFace YPos;
    static HandleFace YNeg;
    static HandleFace ZPos;
    static HandleFace ZNeg;
    static std::array<HandleFace, 6> Faces;
};

enum HandlesType {
    MoveHandles,
    ScaleHandles,
    RotateHandles,
};

class DEF_INST_(abstract) Handles : public Instance {
    AUTOGEN_PREAMBLE
public:
    bool nixAxes = false; // XYZ -> ZXY, used with rotation
    bool active;
    std::weak_ptr<Part> adornee;
    HandlesType handlesType;

    // inline std::weak_ptr<Part> GetAdornee() { return adornee; }
    // inline void SetAdornee(std::weak_ptr<Part> newAdornee) { this->adornee = newAdornee; updateAdornee(); };

    Handles();

    // World-space handles vs local-space handles
    bool worldMode = false;
    CFrame GetCFrameOfHandle(HandleFace face);
    CFrame PartCFrameFromHandlePos(HandleFace face, Vector3 newPos);
    Vector3 HandleSize(HandleFace face);
    std::optional<HandleFace> RaycastHandle(rp3d::Ray ray);

    static inline std::shared_ptr<Handles> New() { return std::make_shared<Handles>(); };
};